@@ -44,7 +44,7 @@ _BEGIN_STD_C
 /* Natural log of 2 */
 #define _M_LN2        0.693147180559945309417
 
-#if __GNUC_PREREQ (3, 3)
+#if __GNUC_PREREQ (3, 3) || defined(__clang__) || defined(__COMPCERT__)
  /* gcc >= 3.3 implicitly defines builtins for HUGE_VALx values.  */
 
 # ifndef HUGE_VAL
@@ -220,17 +220,27 @@ extern int isnan (double);
 # define MATH_ERREXCEPT 2
 #endif
 #ifndef math_errhandling
-# define math_errhandling MATH_ERRNO
+# ifdef _IEEE_LIBM
+#  define _MATH_ERRHANDLING_ERRNO 0
+# else
+#  define _MATH_ERRHANDLING_ERRNO MATH_ERRNO
+# endif
+# ifdef _SUPPORTS_ERREXCEPT
+#  define _MATH_ERRHANDLING_ERREXCEPT MATH_ERREXCEPT
+# else
+#  define _MATH_ERRHANDLING_ERREXCEPT 0
+# endif
+# define math_errhandling (_MATH_ERRHANDLING_ERRNO | _MATH_ERRHANDLING_ERREXCEPT)
 #endif
 
-extern int __isinff (float x);
-extern int __isinfd (double x);
-extern int __isnanf (float x);
-extern int __isnand (double x);
-extern int __fpclassifyf (float x);
-extern int __fpclassifyd (double x);
-extern int __signbitf (float x);
-extern int __signbitd (double x);
+extern int __isinff (float);
+extern int __isinfd (double);
+extern int __isnanf (float);
+extern int __isnand (double);
+extern int __fpclassifyf (float);
+extern int __fpclassifyd (double);
+extern int __signbitf (float);
+extern int __signbitd (double);
 
 /* Note: isinf and isnan were once functions in newlib that took double
  *       arguments.  C99 specifies that these names are reserved for macros
@@ -239,7 +249,7 @@ extern int __signbitd (double x);
  *       taking double arguments still exist for compatibility purposes
  *       (prototypes for them are earlier in this header).  */
 
-#if __GNUC_PREREQ (4, 4)
+#if __GNUC_PREREQ (4, 4) || defined(__clang__)
   #define fpclassify(__x) (__builtin_fpclassify (FP_NAN, FP_INFINITE, \
 						 FP_NORMAL, FP_SUBNORMAL, \
 						 FP_ZERO, __x))
@@ -256,7 +266,7 @@ extern int __signbitd (double x);
 #else
   #define fpclassify(__x) \
 	  ((sizeof(__x) == sizeof(float))  ? __fpclassifyf(__x) : \
-	  __fpclassifyd(__x))
+	  __fpclassifyd((double) (__x)))
   #ifndef isfinite
     #define isfinite(__y) \
 	    (__extension__ ({int __cy = fpclassify(__y); \
@@ -279,34 +289,34 @@ int __issignaling(double d);
 int __issignalingl(long double d);
 #define issignaling(__x)						\
 	((sizeof(__x) == sizeof(float))  ? __issignalingf(__x) :	\
-	 (sizeof(__x) == sizeof(double)) ? __issignaling (__x) :	\
-	 __issignalingl(__x))
+	 (sizeof(__x) == sizeof(double)) ? __issignaling ((double) (__x)) :	\
+	 __issignalingl((long double) (__x)))
 #else
 #ifdef _DOUBLE_IS_32BITS
 #define issignaling(__x) __issignalingf((float) (__x))
 #else
 #define issignaling(__x)						\
 	((sizeof(__x) == sizeof(float))  ? __issignalingf(__x) :	\
-	 __issignaling (__x))
+	 __issignaling ((double) (__x)))
 #endif /* _DOUBLE_IS_32BITS */
 #endif /* _HAVE_LONG_DOUBLE */
 #endif
 
 #if __GNUC_PREREQ (4, 0)
   #if defined(_HAVE_LONG_DOUBLE)
-    #define signbit(__x) \
-	    ((sizeof(__x) == sizeof(float))  ? __builtin_signbitf(__x) : \
-	     (sizeof(__x) == sizeof(double)) ? __builtin_signbit (__x) : \
-					       __builtin_signbitl(__x))
+    #define signbit(__x)							\
+	    ((sizeof(__x) == sizeof(float))  ? __builtin_signbitf(__x) :	\
+	     ((sizeof(__x) == sizeof(double)) ? __builtin_signbit ((double)(__x)) : \
+	      __builtin_signbitl((long double)(__x))))
   #else
-    #define signbit(__x) \
-	    ((sizeof(__x) == sizeof(float))  ? __builtin_signbitf(__x) : \
-					       __builtin_signbit (__x))
+    #define signbit(__x)							\
+	    ((sizeof(__x) == sizeof(float))  ? __builtin_signbitf(__x) :	\
+	     __builtin_signbit ((double) (__x)))
   #endif
 #else
   #define signbit(__x) \
 	  ((sizeof(__x) == sizeof(float))  ?  __signbitf(__x) : \
-		  __signbitd(__x))
+	                              __signbitd((double) (__x)))
 #endif
 
 #if __GNUC_PREREQ (2, 97)
@@ -558,9 +568,7 @@ extern float dremf (float, float);
 #ifdef __CYGWIN__
 extern float dreml (long double, long double);
 #endif /* __CYGWIN__ */
-extern double gamma_r (double, int *);
 extern double lgamma_r (double, int *);
-extern float gammaf_r (float, int *);
 extern float lgammaf_r (float, int *);
 #endif
 
@@ -649,24 +657,10 @@ extern NEWLIB_THREAD_LOCAL int signgam;
 #define M_LOG2_E        _M_LN2
 #define M_INVLN2        1.4426950408889633870E0  /* 1 / log(2) */
 
-/* Global control over fdlibm error handling.  */
-
-enum __fdlibm_version
-{
-  __fdlibm_ieee = -1,
-  __fdlibm_posix
-};
-
-#define _LIB_VERSION_TYPE enum __fdlibm_version
-#define _LIB_VERSION __fdlib_version
-
-extern __IMPORT _LIB_VERSION_TYPE _LIB_VERSION;
-
-#define _IEEE_  __fdlibm_ieee
-#define _POSIX_ __fdlibm_posix
-
 #endif /* __BSD_VISIBLE */
 
+#include <machine/math.h>
+
 _END_STD_C
 
 #ifdef __FAST_MATH__
