@@ -91,8 +91,11 @@ _fseeko64_r (struct _reent *ptr,
      int whence)
 {
   _fpos64_t (*seekfn) (struct _reent *, void *, _fpos64_t, int);
-  _fpos64_t target, curoff;
+  _fpos64_t curoff;
+#if _FSEEK_OPTIMIZATION
+  _fpos64_t target;
   size_t n;
+#endif
 
   int havepos;
 
@@ -311,20 +314,20 @@ _fseeko64_r (struct _reent *ptr,
       fp->_p += n;
       fp->_r -= n;
     }
-  _newlib_flockfile_end(fp);
+  _newlib_flockfile_exit(fp);
   return 0;
 
   /*
    * We get here if we cannot optimise the seek ... just
    * do it.  Allow the seek function to change fp->_bf._base.
    */
+dumb:
 #endif
 
-dumb:
   if (_fflush_r (ptr, fp)
       || seekfn (ptr, fp->_cookie, offset, whence) == POS_ERR)
     {
-      _funlockfile(fp);
+      _newlib_flockfile_exit(fp);
       return EOF;
     }
   /* success: clear EOF indicator and discard ungetc() data */
@@ -334,7 +337,7 @@ dumb:
   fp->_r = 0;
   /* fp->_w = 0; *//* unnecessary (I think...) */
   fp->_flags &= ~__SEOF;
-  _funlockfile(fp);
+  _newlib_flockfile_end (fp);
   return 0;
 }
 
