@@ -333,7 +333,7 @@ lo0bits (__ULong *y)
     {
       k++;
       x >>= 1;
-      if (!x & 1)
+      if (!(x & 1))
 	return 32;
     }
   *y = x;
