@@ -94,48 +94,46 @@ typedef double printf_float_t;
 # define PRINTF_LONGLONG	((PRINTF_LEVEL >= PRINTF_FLT) || defined(_WANT_IO_LONG_LONG))
 #endif
 
-#if PRINTF_LONGLONG
+#ifdef PRINTF_LONGLONG
 typedef unsigned long long ultoa_unsigned_t;
 typedef long long ultoa_signed_t;
 #define SIZEOF_ULTOA __SIZEOF_LONG_LONG__
 #define PRINTF_BUF_SIZE 22
-#define arg_to_t(flags, _s_)	({				\
-	    _s_ long long __v__;				\
-	    if ((flags) & FL_LONG) {				\
-		if ((flags) & FL_REPD_TYPE)			\
-		    __v__ = va_arg(ap, _s_ long long);		\
-		else						\
-		    __v__ = va_arg(ap, _s_ long);		\
-	    } else if ((flags) & FL_SHORT) {			\
-		if ((flags) & FL_REPD_TYPE)			\
-		    __v__ = (_s_ char) va_arg(ap, _s_ int);	\
-		else						\
-		    __v__ = (_s_ short) va_arg(ap, _s_ int);	\
-	    } else {						\
-		__v__ = va_arg(ap, _s_ int);			\
-	    }							\
-	    __v__;						\
-	})
+#define arg_to_t(flags, _s_, _result_)	{				\
+	    if ((flags) & FL_LONG) {					\
+		if ((flags) & FL_REPD_TYPE)				\
+		    *(_result_) = va_arg(ap, _s_ long long);		\
+		else							\
+		    *(_result_) = va_arg(ap, _s_ long);			\
+	    } else if ((flags) & FL_SHORT) {				\
+		if ((flags) & FL_REPD_TYPE)				\
+		    *(_result_) = (_s_ char) va_arg(ap, _s_ int);	\
+		else							\
+		    *(_result_) = (_s_ short) va_arg(ap, _s_ int);	\
+	    } else {							\
+		*(_result_) = va_arg(ap, _s_ int);			\
+	    }								\
+	}
 #else
 typedef unsigned long ultoa_unsigned_t;
 typedef long ultoa_signed_t;
 #define SIZEOF_ULTOA __SIZEOF_LONG__
 #define PRINTF_BUF_SIZE 11
-#define arg_to_t(flags, _s_)	({				\
-	    _s_ long __v__;					\
-	    if ((flags) & FL_LONG) {				\
-		__v__ = va_arg(ap, _s_ long);			\
-	    } else if ((flags) & FL_SHORT) {			\
-		__v__ = (_s_ short) va_arg(ap, _s_ int);	\
-	    } else {						\
-		__v__ = va_arg(ap, _s_ int);			\
-	    }							\
-	    __v__;						\
-	})
+#define arg_to_t(flags, _s_, _result_)	{				\
+	    if ((flags) & FL_LONG) {					\
+		*(_result_) = va_arg(ap, _s_ long);			\
+	    } else if ((flags) & FL_SHORT) {				\
+		*(_result_) = (_s_ short) va_arg(ap, _s_ int);		\
+	    } else {							\
+		*(_result_) = va_arg(ap, _s_ int);			\
+	    }								\
+	}
 #endif
 
-#define arg_to_unsigned(flags) arg_to_t(flags, unsigned)
-#define arg_to_signed(flags) arg_to_t(flags, signed)
+// At the call site the address of the result_var is taken (e.g. "&ap")
+// That way, it's clear that these macros *will* modify that variable
+#define arg_to_unsigned(flags, result_var) arg_to_t((flags), unsigned, (result_var))
+#define arg_to_signed(flags, result_var) arg_to_t((flags), signed, (result_var))
 
 #include "ultoa_invert.c"
 
@@ -148,7 +146,6 @@ typedef long ultoa_signed_t;
 #define FL_NEGATIVE	0x40
 #define FL_LONG 	0x80
 
-
 int
 vfprintf (FILE * stream, const char *fmt, va_list ap)
 {
@@ -220,7 +217,8 @@ vfprintf (FILE * stream, const char *fmt, va_list ap)
 	}
 
 	if (c == 'd' || c == 'i') {
-	    ultoa_signed_t x = arg_to_signed(flags);
+	    ultoa_signed_t x;
+	    arg_to_signed(flags, &x);
 
 	    flags &= ~FL_ALT;
 	    if (x < 0) {
@@ -253,7 +251,11 @@ vfprintf (FILE * stream, const char *fmt, va_list ap)
 		flags |= FL_ALTHEX;
 	        base = 16 | XTOA_UPPER;
 	      ultoa:
-		c = __ultoa_invert (arg_to_unsigned(flags), (char *)buf, base) - (char *)buf;
+		{
+		    ultoa_unsigned_t x;
+		    arg_to_unsigned(flags, &x);
+		    c = __ultoa_invert (x, (char *)buf, base) - (char *)buf;
+		}
 		break;
 
 	      default:
@@ -760,7 +762,8 @@ int vfprintf (FILE * stream, const char *fmt, va_list ap)
 	}
 
 	if (c == 'd' || c == 'i') {
-	    ultoa_signed_t x = arg_to_signed(flags);
+	    ultoa_signed_t x;
+	    arg_to_signed(flags, &x);
 
 	    flags &= ~(FL_NEGATIVE | FL_ALT);
 	    if (x < 0) {
@@ -774,7 +777,8 @@ int vfprintf (FILE * stream, const char *fmt, va_list ap)
 		c = __ultoa_invert (x, (char *)buf, 10) - (char *)buf;
 	} else {
 	    int base;
-	    ultoa_unsigned_t x = arg_to_unsigned(flags);
+	    ultoa_unsigned_t x;
+	    arg_to_unsigned(flags, &x);
 
 	    flags &= ~(FL_PLUS | FL_SPACE);
 
@@ -881,4 +885,12 @@ int vfprintf (FILE * stream, const char *fmt, va_list ap)
 #undef my_putc
 }
 
+#ifndef vfprintf
+#ifdef HAVE_ALIAS_ATTRIBUTE
+__strong_reference(vfprintf, __d_vfprintf);
+#else
+int __d_vfprintf (FILE * stream, const char *fmt, va_list ap) { return vfprintf(stream, fmt, ap); }
+#endif
+#endif
+
 #endif	/* PRINTF_LEVEL > PRINTF_MIN */
