@@ -49,14 +49,7 @@ vsnprintf(char *s, size_t n, const char *fmt, va_list ap)
 	else
 		n--;
 
-	struct __file_str f = {
-		.file = {
-			.flags = __SWR | __SSTR,
-		},
-		.buf = s,
-		.len = 0,
-		.size = n			/* -1,0,...32767 */
-	};
+	struct __file_str f = FDEV_SETUP_STRING_WRITE(s, n);
 
 	i = vfprintf(&f.file, fmt, ap);
 
