@@ -15,6 +15,7 @@
 #include <math.h>
 #include <sys/types.h>
 #include <machine/ieeefp.h>
+#include "math_config.h"
 
 /* Most routines need to check whether a float is finite, infinite, or not a
    number, and many need to know whether the result of an operation will
@@ -158,7 +159,9 @@ extern double __ieee754_cosh __P((double));
 extern double __ieee754_fmod __P((double,double));
 extern double __ieee754_pow __P((double,double));
 extern double __ieee754_lgamma __P((double));
-extern double __ieee754_gamma __P((double));
+extern double ___ieee754_lgamma_r __P((double, int *));
+extern double __ieee754_lgamma_r __P((double, int *));
+extern double __ieee754_tgamma __P((double));
 extern double __ieee754_log10 __P((double));
 extern double __ieee754_sinh __P((double));
 extern double __ieee754_hypot __P((double,double));
@@ -177,7 +180,6 @@ extern double __ieee754_scalb __P((double,double));
 #endif
 
 /* fdlibm kernel function */
-extern double __kernel_standard __P((double,double,int));
 extern double __kernel_sin __P((double,double,int));
 extern double __kernel_cos __P((double,double));
 extern double __kernel_tan __P((double,double,int));
@@ -204,7 +206,9 @@ extern float __ieee754_coshf __P((float));
 extern float __ieee754_fmodf __P((float,float));
 extern float __ieee754_powf __P((float,float));
 extern float __ieee754_lgammaf __P((float));
-extern float __ieee754_gammaf __P((float));
+extern float ___ieee754_lgammaf_r __P((float, int *));
+extern float __ieee754_lgammaf_r __P((float, int *));
+extern float __ieee754_tgammaf __P((float));
 extern float __ieee754_log10f __P((float));
 extern float __ieee754_sinhf __P((float));
 extern float __ieee754_hypotf __P((float,float));
@@ -222,7 +226,7 @@ extern float __ieee754_scalbf __P((float,int));
 extern float __ieee754_scalbf __P((float,float));
 #endif
 
-#if !__OBSOLETE_MATH
+#if !__OBSOLETE_MATH_DOUBLE
 /* The new math code does not provide separate wrapper function
    for error handling, so the extern symbol is called directly.
    This is valid as long as there are no namespace issues (the
@@ -231,6 +235,8 @@ extern float __ieee754_scalbf __P((float,float));
 # define __ieee754_exp(x) exp(x)
 # define __ieee754_log(x) log(x)
 # define __ieee754_pow(x,y) pow(x,y)
+#endif
+#if !__OBSOLETE_MATH_FLOAT
 # define __ieee754_expf(x) expf(x)
 # define __ieee754_logf(x) logf(x)
 # define __ieee754_powf(x,y) powf(x,y)
@@ -264,10 +270,10 @@ extern int   __kernel_rem_pio2f __P((float*,float*,int,int,int,const __int32_t*)
 
 #ifdef __IEEE_BIG_ENDIAN
 
-typedef union 
+typedef union
 {
-  double value;
-  struct 
+  uint64_t bits;
+  struct
   {
     __uint32_t msw;
     __uint32_t lsw;
@@ -278,10 +284,10 @@ typedef union
 
 #ifdef __IEEE_LITTLE_ENDIAN
 
-typedef union 
+typedef union
 {
-  double value;
-  struct 
+  uint64_t bits;
+  struct
   {
     __uint32_t lsw;
     __uint32_t msw;
@@ -295,7 +301,7 @@ typedef union
 #define EXTRACT_WORDS(ix0,ix1,d)				\
 do {								\
   ieee_double_shape_type ew_u;					\
-  ew_u.value = (d);						\
+  ew_u.bits = asuint64(d);					\
   (ix0) = ew_u.parts.msw;					\
   (ix1) = ew_u.parts.lsw;					\
 } while (0)
@@ -305,7 +311,7 @@ do {								\
 #define GET_HIGH_WORD(i,d)					\
 do {								\
   ieee_double_shape_type gh_u;					\
-  gh_u.value = (d);						\
+  gh_u.bits = asuint64(d);					\
   (i) = gh_u.parts.msw;						\
 } while (0)
 
@@ -314,7 +320,7 @@ do {								\
 #define GET_LOW_WORD(i,d)					\
 do {								\
   ieee_double_shape_type gl_u;					\
-  gl_u.value = (d);						\
+  gl_u.bits = asuint64(d);					\
   (i) = gl_u.parts.lsw;						\
 } while (0)
 
@@ -325,7 +331,7 @@ do {								\
   ieee_double_shape_type iw_u;					\
   iw_u.parts.msw = (ix0);					\
   iw_u.parts.lsw = (ix1);					\
-  (d) = iw_u.value;						\
+  (d) = asdouble(iw_u.bits);					\
 } while (0)
 
 /* Set the more significant 32 bits of a double from an int.  */
@@ -333,9 +339,9 @@ do {								\
 #define SET_HIGH_WORD(d,v)					\
 do {								\
   ieee_double_shape_type sh_u;					\
-  sh_u.value = (d);						\
+  sh_u.bits = asuint64(d);					\
   sh_u.parts.msw = (v);						\
-  (d) = sh_u.value;						\
+  (d) = asdouble(sh_u.bits);					\
 } while (0)
 
 /* Set the less significant 32 bits of a double from an int.  */
@@ -343,37 +349,21 @@ do {								\
 #define SET_LOW_WORD(d,v)					\
 do {								\
   ieee_double_shape_type sl_u;					\
-  sl_u.value = (d);						\
+  sl_u.bits = asuint64(d);					\
   sl_u.parts.lsw = (v);						\
-  (d) = sl_u.value;						\
+  (d) = asdouble(sl_u.bits);					\
 } while (0)
 
 /* A union which permits us to convert between a float and a 32 bit
    int.  */
 
-typedef union
-{
-  float value;
-  __uint32_t word;
-} ieee_float_shape_type;
-
 /* Get a 32 bit int from a float.  */
 
-#define GET_FLOAT_WORD(i,d)					\
-do {								\
-  ieee_float_shape_type gf_u;					\
-  gf_u.value = (d);						\
-  (i) = gf_u.word;						\
-} while (0)
+#define GET_FLOAT_WORD(i,d) ((i) = asuint(d))
 
 /* Set a float from a 32 bit int.  */
 
-#define SET_FLOAT_WORD(d,i)					\
-do {								\
-  ieee_float_shape_type sf_u;					\
-  sf_u.word = (i);						\
-  (d) = sf_u.value;						\
-} while (0)
+#define SET_FLOAT_WORD(d,i) ((d) = asfloat(i))
 
 /* Macros to avoid undefined behaviour that can arise if the amount
    of a shift is exactly equal to the size of the shifted operand.  */
