@@ -42,6 +42,6 @@ clog10(double complex z)
 	rr = cabs(z);
 	p = log10(rr);
 	rr = atan2(cimag(z), creal(z)) * M_IVLN10;
-	w = p + rr * I;
+	w = p + rr * (double complex) I;
 	return w;
 }
