@@ -47,7 +47,8 @@ double mretval = 64;
 int traperror = 1;
 char *mname;
 
-void translate_to (FILE *file,
+void
+translate_to (FILE *file,
 	    double r)
 {
   __ieee_double_shape_type bits;
@@ -55,9 +56,37 @@ void translate_to (FILE *file,
   fprintf(file, "0x%08lx, 0x%08lx", (unsigned long) bits.parts.msw, (unsigned long) bits.parts.lsw);
 }
 
-/* Convert double to float, preserving issignaling status */
-#define to_float(f,d)	do { if (issignaling(d)) (f) = __builtin_nansf(""); else (f) = (d); } while(0)
-#define to_double(d,f)	do { if (issignaling(f)) (d) = __builtin_nans(""); else (d) = (f); } while(0)
+/* Convert double to float, preserving issignaling status and NaN sign */
+
+#define to_float(f,d)	do {					\
+		if (__issignaling(d)) {				\
+			if (signbit(d))				\
+				(f) = -__builtin_nansf("");	\
+			else					\
+				(f) =  __builtin_nansf("");	\
+		} else if (isnan(d)) {				\
+			if (signbit(d))				\
+				(f) = -__builtin_nanf("");	\
+			else					\
+				(f) =  __builtin_nanf("");	\
+		} else						\
+			(f) = (float) (d);			\
+	} while(0)
+
+#define to_double(d,f)	do {					\
+		if (__issignalingf(f)) {				\
+			if (signbit(f))				\
+				(d) = -__builtin_nans("");	\
+			else					\
+				(d) =  __builtin_nans("");	\
+		} else if (isnanf(f)) {				\
+			if (signbit(f))				\
+				(d) = -__builtin_nan("");	\
+			else					\
+				(d) =  __builtin_nan("");	\
+		} else						\
+			(d) = (double) (f);			\
+	} while(0)
 
 int
 ffcheck_id(double is,
@@ -150,11 +179,12 @@ fffcheck_id (float is,
   __ieee_double_shape_type is_double;
   int mag;
   isbits.value = is;
-  is_double.value = is;
+  to_double(is_double.value, is);
 
   correct_double.parts.msw = p->qs[id].msw;
   correct_double.parts.lsw = p->qs[id].lsw;
   to_float(correct.value, correct_double.value);
+//  printf("%s got 0x%08x want 0x%08x\n", name, isbits.p1, correct.p1);
 
   int error_bit = p->error_bit;
 
@@ -167,7 +197,7 @@ fffcheck_id (float is,
   if (isnan(correct.value) && isnan(is)
 #ifndef __i386__
       /* i386 calling convention ends up always converting snan into qnan */
-      && (issignaling(correct.value) == issignaling(is))
+      && (__issignalingf(correct.value) == __issignalingf(is))
 #endif
 	  )
   {
@@ -182,8 +212,8 @@ fffcheck_id (float is,
 	   name,  p->line, mag,
 	   (unsigned long) (uint32_t) correct.p1,
 	   (unsigned long) (uint32_t) isbits.p1,
-	   correct.value,
-	   is,
+	   (double) correct.value,
+	   (double) is,
 	   (unsigned long) (uint32_t) is_double.parts.msw,
 	   (unsigned long) (uint32_t) is_double.parts.lsw);
   }
@@ -233,6 +263,7 @@ int calc;
 extern int reduce;
 
 
+void
 frontline (FILE *f,
        int mag,
        one_line_type *p,
@@ -289,6 +320,7 @@ frontline (FILE *f,
   fprintf(f, ")*/\n");
 }
 
+void
 finish (FILE *f,
        int vector,
        double result,
@@ -305,6 +337,7 @@ finish (FILE *f,
   }
 }
 
+void
 finish2 (FILE *f,
 	 int vector,
 	 double result,
@@ -329,6 +362,7 @@ finish2 (FILE *f,
   }
 }
 
+void
 ffinish (FILE *f,
        int vector,
        float fresult,
@@ -341,10 +375,11 @@ ffinish (FILE *f,
   mag = fffcheck(fresult, p,name,  merror, errno);
   if (vector)
   {
-    frontline(f, mag, p, fresult, merror, errno, args , name);
+    frontline(f, mag, p, (double) fresult, merror, errno, args , name);
   }
 }
 
+void
 ffinish2 (FILE *f,
 	  int vector,
 	  float fresult,
@@ -362,10 +397,10 @@ ffinish2 (FILE *f,
   if (vector)
   {
     __ieee_double_shape_type result2_double;
-    to_double(result2_double.value, fresult2);
+    to_double(result2_double.value, (double) fresult2);
     p->qs[2].msw = result2_double.parts.msw;
     p->qs[2].lsw = result2_double.parts.lsw;
-    frontline(f, mag, p, fresult, merror, errno, args , name);
+    frontline(f, mag, p, (double) fresult, merror, errno, args , name);
   }
 }
 
@@ -376,7 +411,7 @@ in_float_range(double arg)
 {
 	if (isinf(arg) || isnan(arg))
 		return true;
-	return -FLT_MAX <= arg && arg < FLT_MAX;
+	return -(double) FLT_MAX <= arg && arg < (double) FLT_MAX;
 }
 
 void
@@ -504,6 +539,8 @@ run_vector_1 (int vector,
 	 to_float(arga, arg1);
 	 to_float(argb, arg2);
 	 fresult = ((pdblfunc)(func))(arga, argb);
+//	 printf("0x%08x = %s(0x%08x, 0x%08x)\n",
+//		float_bits(fresult), name, float_bits(arga), float_bits(argb));
 	 ffinish(f, vector, fresult, p,args, name);
        }
      }
@@ -578,6 +615,8 @@ test_math (int vector)
   test_atanhf(vector);
   test_ceil(vector);
   test_ceilf(vector);
+  test_copysign(vector);
+  test_copysignf(vector);
   test_cos(vector);
   test_cosf(vector);
   test_cosh(vector);
