@@ -353,6 +353,12 @@ test_ceil(int vector);
 void
 test_ceilf(int vector);
 
+void
+test_copysign(int vector);
+
+void
+test_copysignf(int vector);
+
 void
 test_cos(int vector);
 
