@@ -42,7 +42,7 @@
 #include <stdlib.h>
 
 #ifndef TINY_STDIO
-#define printf_float(x) x
+#define printf_float(x) ((double) (x))
 
 #ifdef _NANO_FORMATTED_IO
 #ifndef NO_FLOATING_POINT
@@ -55,7 +55,9 @@ int (*_reference_scanf_float)() = _scanf_float;
 #endif
 #endif
 
+#if defined(TINY_STDIO) || !defined(NO_FLOATING_POINT)
 static const double test_vals[] = { 1.234567, 1.1, M_PI };
+#endif
 
 int
 main(int argc, char **argv)
@@ -86,12 +88,14 @@ main(int argc, char **argv)
 	}
 	printf ("%g\n", exp(11));
 #endif
+#if defined(TINY_STDIO) || !defined(NO_FLOATING_POINT)
 	sprintf(buf, "%g", printf_float(0.0f));
 	if (strcmp(buf, "0") != 0) {
 		printf("0: wanted \"0\" got \"%s\"\n", buf);
 		errors++;
 		fflush(stdout);
 	}
+#endif
 	for (x = 0; x < 32; x++) {
 		unsigned int v = 0x12345678 >> x;
 		unsigned int r;
@@ -126,6 +130,8 @@ main(int argc, char **argv)
 		for (t = 0; t < sizeof(test_vals)/sizeof(test_vals[0]); t++) {
 #ifdef PICOLIBC_FLOAT_PRINTF_SCANF
 #define float_type float
+#define pow(a,b) powf((float) a, (float) b)
+#define fabs(a) fabsf(a)
 #define scanf_format "%f"
 #if defined(TINY_STDIO) && !defined(_IO_FLOAT_EXACT)
 #define ERROR_MAX 1e-6
@@ -142,14 +148,14 @@ main(int argc, char **argv)
 #endif
 #endif
 
-			float_type v = test_vals[t] * pow(10.0, (float_type) x);
+			float_type v = (float_type) test_vals[t] * pow(10.0, (float_type) x);
 			float_type r;
 			float_type e;
 
 			sprintf(buf, "%.55f", printf_float(v));
 			sscanf(buf, scanf_format, &r);
 			e = fabs(v-r) / v;
-			if (e > ERROR_MAX) {
+			if (e > (float_type) ERROR_MAX) {
 				printf("\tf %3d: wanted %.7e got %.7e (error %.7e, buf %s)\n", x,
 				       printf_float(v), printf_float(r), printf_float(e), buf);
 				errors++;
@@ -160,7 +166,7 @@ main(int argc, char **argv)
 			sprintf(buf, "%.20e", printf_float(v));
 			sscanf(buf, scanf_format, &r);
 			e = fabs(v-r) / v;
-			if (e > ERROR_MAX)
+			if (e > (float_type) ERROR_MAX)
 			{
 				printf("\te %3d: wanted %.7e got %.7e (error %.7e, buf %s)\n", x,
 				       printf_float(v), printf_float(r), printf_float(e), buf);
@@ -172,7 +178,7 @@ main(int argc, char **argv)
 			sprintf(buf, "%.20g", printf_float(v));
 			sscanf(buf, scanf_format, &r);
 			e = fabs(v-r) / v;
-			if (e > ERROR_MAX)
+			if (e > (float_type) ERROR_MAX)
 			{
 				printf("\tg %3d: wanted %.7e got %.7e (error %.7e, buf %s)\n", x,
 				       printf_float(v), printf_float(r), printf_float(e), buf);
